Index: /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.h
===================================================================
--- /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.h	(revision 61429)
+++ /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.h	(revision 61430)
@@ -51,4 +51,7 @@
     /** Constructor, passes @a pMachineLogic and @a uScreenId to the UIMachineWindow constructor. */
     UIMachineWindowFullscreen(UIMachineLogic *pMachineLogic, ulong uScreenId);
+
+    /** Handles Qt change @a pChangeEvent. */
+    void changeEvent(QEvent *pChangeEvent);
 
 #ifdef VBOX_WS_MAC

Index: /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.cpp
===================================================================
--- /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.cpp	(revision 61429)
+++ /trunk/src/VBox/Frontends/VirtualBox/src/runtime/fullscreen/UIMachineWindowFullscreen.cpp	(revision 61430)
@@ -56,4 +56,38 @@
 #endif /* VBOX_WS_MAC */
 {
+}
+
+void UIMachineWindowFullscreen::changeEvent(QEvent *pChangeEvent)
+{
+    /* Overriding changeEvent for controlling mini-tool bar visibilty.
+     * Hiding tool-bar when machine-window is minimized and show it in full-screen: */
+#if defined(Q_WS_WIN) || defined(Q_WS_X11)
+    if (pChangeEvent->type() == QEvent::WindowStateChange)
+    {
+        /* If machine-window is in minimized mode: */
+        if(isMinimized())
+        {
+            /* If there is a mini-toolbar: */
+            if (m_pMiniToolBar)
+            {
+                /* Hide mini-toolbar: */
+                m_pMiniToolBar->hide();
+            }
+        }
+        /* If machine-window is in full-screen mode: */
+        else
+        {
+            /* If there is a mini-toolbar: */
+            if (m_pMiniToolBar)
+            {
+                /* Show mini-toolbar in full-screen mode: */
+                m_pMiniToolBar->showFullScreen();
+            }
+        }
+    }
+#endif /* Q_WS_WIN || Q_WS_X11 */
+
+    /* Call to base-class: */
+    QMainWindow::changeEvent(pChangeEvent);
 }
 
@@ -375,13 +409,4 @@
         !pFullscreenLogic->hasHostScreenForGuestScreen(m_uScreenId))
     {
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
-        /* If there is a mini-toolbar: */
-        if (m_pMiniToolBar)
-        {
-            /* Hide mini-toolbar: */
-            m_pMiniToolBar->hide();
-        }
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
-
         /* Hide window: */
         hide();
@@ -389,6 +414,6 @@
     else
     {
-        /* Ignore if window minimized: */
-        if (isMinimized())
+        /* Ignore if window is minimized and visible: */
+        if (isMinimized() && isVisible())
             return;
 
@@ -422,12 +447,14 @@
         }
 #elif defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
-        /* Show window in fullscreen mode: */
-        showFullScreen();
-
-        /* If there is a mini-toolbar: */
-        if (m_pMiniToolBar)
-        {
-            /* Show mini-toolbar in full-screen mode: */
-            m_pMiniToolBar->showFullScreen();
+        /* If machine-window is in minimized mode: */
+        if (isMinimized())
+        {
+            /* Show window in minimized mode: */
+            showMinimized();
+        }
+        else
+        {
+            /* Show window in fullscreen mode: */
+            showFullScreen();
         }
 #endif /* VBOX_WS_WIN || VBOX_WS_X11 */